"""
/**********************************************************************
FILE     : $RCSfile: icoyaDrive.py,v $
PURPOSE  : icoya2
NOTES    : contains the icoya2 gui content manager gui and its methods
AUTHOR   : Simon Eisenmann et. al.
COPYRIGHT: (c) 2002-2003 by struktur AG. All rights reserved.
         : This software is Open Source and can be used freely under
         : GPL licensing terms.
DATE     : 12FEB2002
REVISION : $Revision: 1.7 $
VERSION  : $Id: icoyaDrive.py,v 1.7 2002/08/15 12:24:33 longsleep Exp $ (Author: $Author: longsleep $)

struktur AG            Phone: +49 711 8966560
Junghansstr. 5         Fax:   +49 711 89665610
70469 Stuttgart        email: info@struktur.de
GERMANY 

http://www.struktur.de
http://www.strukturag.com

**********************************************************************/

icoya2 content manager class for icoyaDrive


 This software is part of icoya, an XML content and 
 collaboration management (CCM) system by struktur AG. 
 See http://www.icoya.org
     http://www.icoya.com
     http://www.icoya.de

 This software has been developed by struktur AG
 http://www.struktur.de
 http://www.strukturag.com

 This software is Open Source and can be used freely under GPL licensing terms.


"""

__doc__ = 'icoyaDrive'
__version__= '$Revision: 1.7 $'[11:-2]


# TODO: clean up this ...
import urllib,time,re,string,sys, types, time
from urllib import urlencode

# zope imports
import Globals
from Globals import package_home
import Acquisition
import AccessControl
from AccessControl import getSecurityManager
from OFS.Traversable import Traversable
from OFS.SimpleItem import SimpleItem

# products and 3rd party imports
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

# icoya imports
from Products.icoyaProject import icoya2

# ---------- implementation goes here  ------------------------

class icoyaDrive(icoya2.icoya2):
    """
    icoya2 class, the content manager gui and its methods
    for icoyaDrive
    """

    meta_type = 'icoyaDrive'
    manage_editForm = PageTemplateFile('www/manage_icoyaDrive', globals())


    def _support(self, caller, icoyaArgs):
        """icoya2 method"""

        # display files from support dir to web
        # TODO: implement ram cache
        # TODO: implement Pdata like wrapper and write to RESPONSE
        
        file=icoyaArgs['support']

        support=caller._getZopeDir()+'/lib/python/Products/icoyaDrive/icoya2/support/'+str(file)
        file=open(support, 'rb', -1)
        binary=file.read()
        file.close()

        return binary


    def _link(self, caller, icoyaArgs):
        """icoya2 method"""

        params={'method': icoyaArgs['method'], 'support': icoyaArgs['support'], 'cmd': icoyaArgs['cmd'], 'mode': icoyaArgs['mode']}
        link=caller.absolute_url() + '/icoyaDrive/?' + urllib.urlencode(params)
            
        return link


    def _display(self, caller, icoyaArgs):
        """used to display browser content"""
        
        # parse the command
        topic,cmd = string.split(icoyaArgs['cmd'],'_')

        # set folder
        if topic == 'ICOYA':
            folder='/'
        else:
            folder='/'+topic+'/'

        # create pt instance and store it in this instance
        # _v_* attributes are volatile and are not synced with ZODB
        attrName='_v_PageTemplateFileInst_' + str(icoyaArgs['cmd'])
        
        # check for cached instance
        if hasattr(self, attrName):
            if callable(getattr(self, attrName, None)):
                #print "returning cached instance from", attrName
                return getattr(self, attrName)
        
        # create new instance
        #print "creating new instance in", attrName
        tmp=PageTemplateFile('icoya2'+folder+icoyaArgs['cmd'], globals())
        setattr(self, attrName, tmp)

        # return it instance
        return getattr(self, attrName)


    def _action(self, caller, icoyaArgs):
        """used to make the things"""
        
        # set cmd
        cmd = str(icoyaArgs['cmd'])
        
        # create action instance and get the cmd function
        actionInst=icoyaAction(self, caller, icoyaArgs)
        actionFunc = getattr(actionInst, cmd, None)
        
        #print "actionFunc", actionFunc

        if actionFunc:
            # call the function; the function returns the next page to be displayed
            cmd = actionFunc()
            return self._display(caller, {'cmd': cmd })
        else:
            # if not found return error Page
            return self.icoya_error(err='No method for command '+str(cmd)+' in icoyaAction', doraise=1)


    def _setResult(self, caller, icoyaArgs=None, msgstr=''):


        if not hasattr(caller, '_iAmFromIcoyaDrive'):
            # old way (icoya2 action class compatibility)

            RESPONSE=icoyaArgs['RESPONSE']
            REQUEST=icoyaArgs['REQUEST']

            params={'method': icoyaArgs['method'], 'cmd': icoyaArgs['cmd'], 'msgstr': msgstr }

            # add some additional params
            try: params['show'] = REQUEST['show']
            except: pass
            try: params['language'] = REQUEST['language']
            except: pass
            try: params['language'] = REQUEST.form['language']
            except: pass
            try: params['method'] = REQUEST['icoyaNextMethod']
            except: pass
            try: params['cmd'] = REQUEST['icoyaNextCmd']
            except: pass

            params=urlencode(params)
            #print "params", params

            # get custom url from request
            url=REQUEST.get('icoyaNextURL')
            if url == None:
                # else set default
                url=caller.absolute_url()

            RESPONSE.redirect(url+'/icoyaDrive/?'+params)
            return 'ICOYA_blank'

        else:

            RESPONSE=caller._RESPONSE
            RESPONSE.setHeader("Content-Type", "text/xml")
            
            xml="""<?xml version='1.0' encoding='UTF-8'?>
                <icoyaDriveResponse>
            """

            vars = dir(caller)
            for attr in ('_REQUEST', '_RESPONSE', '_icoya', '_caller', '_session', '_icoyaArgs', 'cmd', 'f_content'):
                try:
                    vars.remove(attr)
                except:
                    pass
                
            for attr in vars:
                if callable(getattr(caller,attr, '')):
                    vars.remove(attr)
                
            for attr in vars:
                xml += "<"+str(attr)+">"+str(getattr(caller, attr, ''))+"</"+str(attr)+">"
                
            for key in caller._icoyaArgs.keys():
                if key in ('REQUEST', 'RESPONSE'):
                    continue
                xml += "<"+str(key)+">"+str(caller._icoyaArgs.get(key))+"</"+str(key)+">"

            xml += "<callerurl>"+str(caller._caller.absolute_url())+"</callerurl>"

            if 'f_id' in vars:
                # return cid also
                try:
                    cid=getattr(caller._caller, getattr(caller, 'f_id')).getcid()
                    xml += "<f_cid>"+str(cid)+"</f_cid>"
                except:
                    self.icoya_error()

            xml += """
                </icoyaDriveResponse>

            """
            RESPONSE.write(xml)
            return 'ICOYA_blank'


def manage_addicoyaDrive(self, id='icoyaDrive', title='icoyaDrive GUI', REQUEST=None):
    """add icoyaDrive gui """

    id = str(id)
    title=str(title)
    ob=icoyaDrive(id, title)

    # create icoyaDrive object
    id=self._setObject(id, ob)


    if REQUEST is not None:
        return self.manage_main(self, REQUEST)


manage_addicoyaDriveForm = Globals.DTMLFile('www/manage_addicoyaDriveForm', globals())





class icoyaAction(icoya2.icoyaAction):


    _iAmFromIcoyaDrive = 1


    # handler for content editing and creation
    #################################
    def doContent(self, noresponse=0):
        # stores and creates content
        
        # get create function
        editFunc=getattr(self, str(self._caller.meta_type)+'Edit', None)
        if editFunc:
            # call the edit function
            try:
                self.msgstr=editFunc()
            except:
                self.msgstr="failed to store the content"
                self.icoya_error()
        else:
            self.msgstr='no function to edit the content of this object'
        
        # set the result
        #self._icoyaArgs['method']='action'
        #self._icoyaArgs['cmd']='content'

        if not noresponse:        
            self.cmd=self._icoya._setResult(self)
        
        return self.cmd


    # relay add method 
    #################################
    def relayCreate(self, noresponse=0):
        """
        creates relayobjects (publish)
        """
        
        REQUEST=self._REQUEST
        self.msgstr='published your object successfully'
        
        try:
            # get id from request
            newid=REQUEST.get('id', None)
            
            # get resourecID from request
            useResourceID=REQUEST.get('useResourceID', None)
            urlResource=REQUEST.get('url', '')
            
            # parse the resouce.id attribute
            if useResourceID=='true' and urlResource != '':
                try: newid=string.split(urlparse.urlparse(urlResource)[2], '/')[-1]
                except: newid=None
            
            # generate file id if none
            if not newid:
                newid=self._getNewFileId()
                
            # create a mapping from resource cids to relay cids
            self._caller.manage_addProduct['icoyaRelayObject'].manage_addicoyaRelayObjectPublicationAction(id=newid,title=str(REQUEST.get('title')), relaycid=REQUEST.get('relaycid'),version=REQUEST.get('version'),language=str(REQUEST.get('language')), REQUEST=REQUEST )
            currentRelay = getattr(self._caller, newid)

            # add id to action class
            self.f_id=newid
            
            # linking may fail
            # TODO: only paresable files may be parsed here (content_type html / xml ..)
            try: 
                resourceCID = REQUEST.get('relaycid')
                resourceObj = self._caller._geticoyaObject(resourceCID)
                links = resourceObj.parseContent()
                self._doRelayLinking(links, currentRelay)
            except: 
                #self.msgstr="some operations failed while publishing<br/><hr/>\n"+self.icoya_error()
                pass
            
        except:
            self.msgstr='failed to publish this object<br/><hr/>\n'+self.icoya_error()
            # unable to get relay so return to caller
            currentRelay=self._caller

        if not noresponse:
            # set me back to resources mode
            self._session.set('mode',2)
            
            # set the result
            #self._icoyaArgs
            self._icoyaArgs['method']='main'
            self._icoyaArgs['cmd']='edit'
            self.cmd=self._icoya._setResult(currentRelay, self._icoyaArgs, self.msgstr)
        
        return 'ICOYA_BLANK'


    # handler for adding a language
    #################################
    def addLanguage(self, noresponse=0):
        # add a langues to a resource
        language = string.strip(self._REQUEST.form['language'])
        if language is not None and language not in self._caller.languages:
            self._caller.initialize(language)

        #self._icoyaArgs['method']='main'
        #self._icoyaArgs['cmd']='edit'
        if not noresponse:
            self.cmd=self._icoya._setResult(self)
        
        return self.cmd

    
    def setMeta(self, noresponse=0):
        # sets several meta data
        # description, date, category
        
        form=self._REQUEST.form

        # TODO: generic interface to icoya_attrib
        # TODO: handle language depended attributes

        lang = form.get('language')
        if lang is not None and type(lang) in (types.StringType, types.UnicodeType): # is this required here?
            string.strip(lang)
        else: 
            # something failed: bad page template! Do some fixing code here
            lang = self._caller._getMyConfig().get('defaultLanguage')

        # set description/keywords/category if present
            
        # update meta data
        update=0
        
        description = form.get('description')
        if description is not None:
            self._caller.icoya_attrib('setDescription', self._caller, description, lang)
            update=1
        keywords = form.get('keywords')
        if keywords is not None:
            self._caller.icoya_attrib('setKeywords', self._caller, keywords, lang)
            update=1
        category = form.get('category')
        if category is not None:
            self._caller.icoya_attrib('setCategory', self._caller, category, lang)
            update=1
        
        try:
            if update and getattr(self._caller, 'isIcoyaResource', None):
                # update linked relays with latest version
                for pub in self._caller.publocations:
                    ob=self._caller._geticoyaObject(pub)
                    if ob:
                        if ob._version=='latest':
                            ob.do_reindex_object()
        except:
            self.msgstr=self.icoya_error()

        # set publication properties
        if self._caller.meta_type == 'icoyaRelayObject':
            #TODO: change to icoyaAttrib
            version = form.get('version')
            self._caller._version = version
            self._caller._language = lang
            self._caller.setPublicationProperties(form)
            self._caller.checkindexingofthisobject()

        # set icoya custom fields
        self._setCustomDataFromRequestForm(self._caller, form, lang)

        # set response
        return self.cmd    
        

    # hander for icoyaCMLLine interface
    #####################################
    def icoyaCMDLineDoContent(self):
        """
        adds content and language trees
        """
        REQUEST=self._REQUEST
        cid=REQUEST.get('cid', None)

        newcaller=None
        if not cid: self.msgstr='no cid was given'
        else: newcaller=self._caller.getObjectByID(cid)
                                
        if newcaller==None:
            self.msgstr='unable to find the object with cid %s' % (cid)

        else:
            self._caller=newcaller
            try:
                language=self._REQUEST.get('_language')
                self._caller._setRequestedVersion(self._REQUEST, kw={'language': language})
            except:
                pass
            self.addLanguage(noresponse=1)
            self.doContent(noresponse=1)
            self.setMeta(noresponse=1)

        self.cmd=self._icoya._setResult(self)
        return self.cmd

    
    def icoyaCMDLineCreate(self):
        """
        creates objects
        """
        REQUEST=self._REQUEST

        # get path or cid
        destpath=REQUEST.get('destpath', None)
        destcid=REQUEST.get('destcid', None)

        # get caller object
        newcaller=None
        if destpath: newcaller=self._caller.unrestrictedTraverse(destpath)
        else: newcaller=self._caller.getObjectByID(destcid)

        if not newcaller:
            self.msgstr='unable to find the destination folder'
        else:
            self._caller=newcaller
            try:
                language=self._REQUEST.get('_language')
                self._caller._setRequestedVersion(self._REQUEST, kw={'language': language})
            except:
                pass
            self.defaultCreate(noresponse=1)
            self.setMeta(noresponse=1)            

        self.cmd=self._icoya._setResult(self)
        return self.cmd

    
    def icoyaCMDLinePublish(self):
        """
        creates objects
        """
        REQUEST=self._REQUEST

        # get path or cid
        destpath=REQUEST.get('destpath', None)
        destcid=REQUEST.get('destcid', None)

        # get caller object
        newcaller=None
        resourceObject=None
        if destpath: newcaller=self._caller.unrestrictedTraverse(destpath)
        else: newcaller=self._caller.getObjectByID(destcid)
   
        if not newcaller:
            self.msgstr='unable to find the destination folder'
        else:
            # get object to publish
            relaycid=REQUEST.get('relaycid', None)
            if relaycid: resourceObject=self._caller.getObjectByID(relaycid)

        if not resourceObject:
            self.msgstr='unable to find the requested resource object'
        else:
            # got resourceObject and destination Folder -> publish
            self._caller=newcaller
            self.relayCreate(noresponse=1)

        self.cmd=self._icoya._setResult(self)
        return self.cmd


    # handler for objects creation
    #################################
    def defaultCreate(self, noresponse=0):
        """creates objects"""
        
        # get request
        REQUEST=self._REQUEST
        
        self.msgstr='created your object'
        
        # set file vars
        self.f_type = str(REQUEST.get('type'))
        self.f_title = str(REQUEST.get('title'))
        self.f_content = None
        id=REQUEST.get('id', None)
        if id == None:
            # no id given so generate one
            id=self._getNewFileId()
        self.f_id=str(id)
        self.f_name=None
        
        # discover the filetitle and load the content
        uploadfile=REQUEST.get('uploadfile')
        if uploadfile != None:
            filename=uploadfile.filename
            self.f_title=filename[max(string.rfind(filename, '/'),
              string.rfind(filename, '\\'),
              string.rfind(filename, ':'),
              )+1:]
                
            # set the discovered values
            self.f_content=uploadfile
            self.f_name=filename
        
        # set default title
        if self.f_title == None:
            self.f_title = 'no title'
        
        # get create function
        addFunc=getattr(self, self.f_type+'Add', None)
        if addFunc:
            # call the add function
            try:
                res=addFunc()
            except:
                self.msgstr="failed to create a object of type "+self.f_type
                self.icoya_error()
        else:
            self.msgstr='no function to create this type of object'
 
        # set the result
        #self._icoyaArgs['method']='main'
        #if self.f_type != 'icoyaLiveFolder':
        #    self._icoyaArgs['cmd']='icoyaOverviewResources'
        #else:
        #    self._icoyaArgs['cmd']='icoyaOverviewWeb'

        if not noresponse:
            self.cmd=self._icoya._setResult(self)
        
        return self.cmd

    
